lappend auto_path [file join $itk_path tclX]
lappend auto_path [file join $itk_path itkObjC.bundle]

if {[info exists argc] && $argc == 2 && [lindex $argv 0] == "-c"} {
    set result [catch [lindex $argv 1] msg]
    if {$result} {
    puts "Error code $result: $msg"
    }
    exit $result
}

proc man:path {subject} {
    global tcl_platform env
    if {$tcl_platform(platform) == "windows"} {
        set localFolder $env(NEXT_ROOT)/LocalLibrary
		set systemFolder $env(NEXT_ROOT)/NextLibrary
    } elseif {$tcl_platform(platform) == "OPENSTEP"} {
        set localFolder /LocalLibrary
		set systemFolder /NextLibrary
    } else {
        set localFolder /Local/Library
		set systemFolder /System/Library
	}
    if {$subject == ""} {
        return $localFolder/Joy/Documentation/CommandReference.html
    } else {
        set docFolder $localFolder/Joy/Documentation/Commands
        set file $docFolder/Tcl/TclCmd/$subject.html
        if {[file exists $file]} {
            return $file
        }
        set file $docFolder/TclX/$subject.txt.html
        if {[file exists $file]} {
            return $file
        }
        set file $docFolder/Joy/$subject.html
        if {[file exists $file]} {
            return $file
        }
        if {[string first objc:: $subject] == 0} {
            set file $docFolder/ObjectiveC/[string range $subject 6 end].html
            if {[file exists $file]} {
                return $file
            }
        }
        if {[string first objc: $subject] == 0} {
            set file $docFolder/ObjectiveC/[string range $subject.html 5 end]
            if {[file exists $file]} {
                return $file
            }
        }
        if {[string first joy:: $subject] == 0} {
            set file $docFolder/Joy/[string range $subject.html 5 end]
            if {[file exists $file]} {
                return $file
            }
        }
        if {[lsearch "java java::new java::call java::defineclass java::field java::instanceof java::prop java::info java::null" $subject] != -1} {
            return $docFolder/Java/TclCmd/java.htm
        }
        if {$subject == "java::load"} {
            return $docFolder/Java/TclCmd/javaload.htm
        }
        if {[lsearch "java::bind java::event java::throw" $subject] != -1} {
            return $docFolder/Java/TclCmd/javabind.htm
        }
        foreach framework {AppKit Foundation EOAccess EOControl EOInterface WebObjects} {
            foreach extension {rtf rtfd html pdf hlp} {
                foreach appendix {"" ClassCluster} {
                    foreach refPlace {"" Reference/Classes Reference/ObjC_classic/Classes Reference/ObjC_Classic/Classes} {
                        set file $systemFolder/Frameworks/$framework.framework/Resources/English.lproj/Documentation/$refPlace/$subject$appendix.$extension
                        if {[file exists $file]} {
                            return $file
                        }
                    }
                }
            }
        }
        return ""
    }
}

proc man {{subject ""}} {
    global env
    set path [man:path $subject]
    if {$path == ""} {
        error "No manual entry for $subject"
    }
    if {![[NSWorkspace sharedWorkspace] openFile: [@ $path]] && ![[NSWorkspace sharedWorkspace] openFile: [@ $path] withApplication: [@ $env(NEXT_ROOT)/NextDemos/TextEdit.app/TextEdit]]} {
        error "Could not open $path"
    }
}
