#import <AppKit/AppKit.h>
#import <Foundation/Foundation.h>
#import <itkTcl/itkTcl.h>
#import <itkTcl/itkObjC.h>
#import "ITKCommandTextView.h"

@interface ITKTclInterp : NSObject
{
    Tcl_Interp		*interp;
    NSString		*initFile;
    unsigned		mainInterpProxy : 1;
    unsigned		browserLoaded : 1;
    NSMapTable		*timedEvals;
    ITKCommandTextView	*commandText;
    int			debugLevel;
}

// The first ITKTclInterp created by the application.

+ (ITKTclInterp *) mainInterp;

// Inform ITKTclInterp about the currently loading nib file.

+ (NSString *) nibFilePath;
+ (void) setNibFilePath: (NSString *) path;

// Access methods

- (NSString *) initFile;
- (void) setInitFile: (NSString *) anInitFile;
- (BOOL) isMainInterpProxy;
- (void) setMainInterpProxy: (BOOL) flag;

// Tcl wrapper methods

- (TclObj) source: (NSString *) file;

- (TclObj) eval: (TclObj) code;
- (TclObj) globEval: (TclObj) code;
- (TclObj) setVar: (TclObj) var to: (TclObj) value flags: (int) flags;
- (TclObj) setVar: (TclObj) var
	    index: (TclObj) index
	       to: (TclObj) value
	    flags: (int) flags;
- (TclObj) getVar: (TclObj) var flags: (int) flags;
- (TclObj) getVar: (TclObj) var index: (TclObj) index flags: (int) flags;
- (int) unsetVar: (TclObj) var flags: (int) flags;
- (int) unsetVar: (TclObj) var index: (TclObj) index flags: (int) flags;
- (int) traceVar: (TclObj) var
	   flags: (int) flags
	    proc: (Tcl_VarTraceProc *) proc
      clientData: (ClientData) clientData;
- (int) traceVar: (TclObj) var
	   index: (TclObj) index
	   flags: (int) flags
	    proc: (Tcl_VarTraceProc *) proc
      clientData: (ClientData) clientData;
- (void) untraceVar: (TclObj) var
	      flags: (int) flags
	       proc: (Tcl_VarTraceProc *) proc
	 clientData: (ClientData) clientData;
- (void) untraceVar: (TclObj) var
	      index: (TclObj) index
	      flags: (int) flags
	       proc: (Tcl_VarTraceProc *) proc
	 clientData: (ClientData) clientData;

// Timed evals: All of these methods return a token (id) that can be passed to
// removeTimedEval: to cancel the request.

- eval: (TclObj) code after: (double) seconds;
- eval: (TclObj) code after: (double) seconds modes: (NSArray *) modes;
- eval: (TclObj) code every: (double) seconds;
- eval: (TclObj) code every: (double) seconds modes: (NSArray *) modes;
- eval: (TclObj) code whenChannelIsReady: (TclObj) channel;
- eval: (TclObj) code whenChannelIsReady: (TclObj) channel
				   modes: (NSArray *) modes;
- evalAfterThisEvent: (TclObj) code order: (unsigned) order;
- evalAfterThisEvent: (TclObj) code
	       order: (unsigned) order
	       modes: (NSArray *) modes;
- evalAfterEveryEvent: (TclObj) code order: (unsigned) order;
- evalAfterEveryEvent: (TclObj) code
		order: (unsigned) order
		modes: (NSArray *) modes;
- evalWhenDeleted: (TclObj) code;

- (void) removeTimedEval: token;

#if 0
// NIB file loading

- (void) loadNibNamed: (NSString *) file;
#endif

// Command window stuff

- (ITKCommandTextView *) commandTextView;
- (NSWindow *) commandWindow;
- (void) showCommandWindow: sender;
- (void) clearCommandWindow: sender;
- (void) saveCommands: sender;
- (NSString *) commandWindowTitle;
- (NSString *) commandPrompt;

@end
