#######################################################################
# Patch to enable Drag'n Drop of file-names and strings into the shell.
#######################################################################

objc:teach ITKCommandTextView {

- initWithFrame: {NSRect frame} {
    former initWithFrame: $frame
    $self registerForDraggedTypes: \
	[NSArray arrayWithObjects: \
		[NSFilenamesPboardType] [NSStringPboardType] nil]
    return $self
}

# Check, if source-type OK and move-mouse-pointer.
- uint draggingEntered: {id sender} {
    set types [[$sender draggingPasteboard] types]
    if {[$types containsObject: [NSFilenamesPboardType]] ||
	[$types containsObject: [NSStringPboardType]]} {
        if {[$sender draggingSourceOperationMask] & [NSDragOperationGeneric]} {
            return [NSDragOperationGeneric]
        }
    }
    return [NSDragOperationNone]
}

# Reflect changes in mouse-pointer-style.
- uint draggingUpdated: {id sender} {
    if {[$sender draggingSourceOperationMask] & [NSDragOperationGeneric]} {
        return [NSDragOperationGeneric]
    }
    return [NSDragOperationNone]
}

- BOOL prepareForDragOperation: {id sender} {
    return [YES]
}

# Do the copying
- BOOL performDragOperation: {id sender} {
    set pboard [$sender draggingPasteboard]
    if {[[$pboard types] containsObject: [NSFilenamesPboardType]]} {
        # Have to split and join for backslash to slash replacement on Windows
        objc:foreach filename \
		     [$pboard propertyListForType: [NSFilenamesPboardType]] {
	    lappend data [eval file join [file split [* $filename]]]
        }
    } elseif {[[$pboard types] containsObject: [NSStringPboardType]]} {
	# convert to list for correct treatment of special characters (e.g. \)
        lappend data [* [$pboard stringForType: [NSStringPboardType]]]
    }
    set data [@ $data]
    if {[$self shouldChangeTextInRange: [$self rangeForUserTextChange] \
                     replacementString: $data]} {
        $self insertText: $data
        $self didChangeText
    }
    return [YES]
}

# Necessary to avoid warning on Console.
- void concludeDragOperation: {id sender} {
}

}

# Use two point larger Courier-Bold if a bold Ohlfs is requested
objc:teach NSFontManager {
    - convertWeight: {BOOL increaseFlag} ofFont: aFont {
        if {$increaseFlag && [* [$aFont familyName]] == "Ohlfs"} {
	    return [NSFont fontWithName: [@ Courier-Bold] \
				   size: [expr [$aFont pointSize] + 2]]
	}
	former convertWeight: $increaseFlag ofFont: $aFont
    }
}
