#--------------------------------------------------------------------------
#  JoyTerminal.nib/Interpreter.tcl
#  Copyright (c) 1997, AAA+ Software (info@aaa-plus.com)
#  All rights reserved.
#  Author: W. Pitzeier
#
#  You may freely copy, distribute and reuse the code in this example.
#  AAA+ disclaims any warranty of any kind, expressed or implied,
#  as to its fitness for any particular use.
#--------------------------------------------------------------------------

objc:teach $NSApp {

    # open new joy command window
    - void newCommandWindow: sender {
        set interp [[ITKTclInterp alloc] init]
        set window [$interp commandWindow]

        # normally a Joy command window does not have a menu bar on
        # the Windows platform, but this application consists of command 
        # windows only, so we need it.  Has no effect on Next and Rhapsody.
        $window setMenu: [$self mainMenu]
        
        $window setExcludedFromWindowsMenu: [NO]

        set keyWindow [$self keyWindow]
        if {$keyWindow == "nil"} {
            # center first window
            $window center
        } else {
            # cascade additional windows
            set frame [$keyWindow frame]
            lassign $frame origin size
            lassign $origin originX originY
            lassign $size width height
            set maxY [expr $originY + $height]
            set point "$originX $maxY"
            set point [$window cascadeTopLeftFromPoint: $point]
            $window setFrameTopLeftPoint: $point
        }

        $interp showCommandWindow: $sender
    }

    # called automatically when loading InfoPanel.nib
    - void setInfoPanel: panel {
        global infoPanel
        set infoPanel $panel
    }

    - void showInfoPanel: sender {
        global infoPanel
        if {![info exists infoPanel]} {
            NSBundle loadNibNamed: [@ InfoPanel] owner: $self
            $infoPanel center
        }
        $infoPanel makeKeyAndOrderFront: $sender
    }
    
    # opens this source file
    - void openSource: sender {
        set path [[[NSBundle mainBundle] pathForResource: [@ JoyTerminal] ofType: [@ nib]] stringByAppendingPathComponent: [@ Interpreter.tcl]]
        set workspace [NSWorkspace sharedWorkspace]
        if {![$workspace openFile: $path]} {
            # If no application opens a file with that extension we open
            # it with TextEdit as default application.  This only
            # happens on Windows.
            global env
            $workspace openFile: $path withApplication: [@ $env(NEXT_ROOT)/NextDemos/TextEdit.app/TextEdit]
        }
    }
    
    # opens all nib files of this application (for all languages)
    - void openNibs: sender {
        foreach language [portableGlob [* [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent: [@ *.lproj]]]] {
            foreach nibFile [portableGlob [* [[@ $language] stringByAppendingPathComponent: [@ *.nib]]]] {
                # we need to open the file inside the nib since Windows
                # can not open a folder with an application.
                [NSWorkspace sharedWorkspace] openFile: [[@ $nibFile] stringByAppendingPathComponent: [@ objects.nib]]
            }
        }
    }

    - void closeModalPanel: sender {
         $self stopModalWithCode: [$sender tag]
    }
}


objc:teach ITKTclInterp {

    # source tcl commands into the key command window
    - void sourceCommands: sender {
        set types [NSArray arrayWithObjects: [@ tcl] [@ joy] nil]
        set openPanel [NSOpenPanel openPanel]
        $openPanel setAllowsMultipleSelection: [YES]
        if {[$openPanel runModalForTypes: $types] == [NSOKButton]} {
            objc:foreach filename [$openPanel filenames] {
                $self source: $filename
            }
        }
    }

    # free memory of closed command window's interpreter
    - BOOL windowShouldClose: window {
        if {[set result [former windowShouldClose: $window]]} {
            $self autorelease
        }
        return $result
    }

    # print the key window's content
    - void printWindow: sender {
        [$self commandTextView] print: $sender
    }

    # called automatically when loading SetTitlePanel.nib
    - void setTitleField: field {
        global titleField
        set titleField $field
    }

    # change the title of the key window
    - void setWindowTitle: sender {
        global titleField titlePanel NSApp
        if {![info exists titleField]} {
            NSBundle loadNibNamed: [@ SetTitlePanel] owner: $self
            set titlePanel [[$titleField controlView] window]
            $titlePanel center
        }
        set window [$self commandWindow]
        $titleField setStringValue: [$window title]
        set tag [$NSApp runModalForWindow: $titlePanel]
        $titlePanel close
        if {$tag} {
            $window setTitle: [$titleField stringValue]
        }
    }
}

# portable version of glob needed for windows,
# since glob only allows slash path separators
proc portableGlob pattern {
    glob [join [split $pattern \\] /]
}

# application initialization
$NSApp newCommandWindow: $self
