###############################################################################
#
# (C) 1994 AAA+ Software Forschungs- & Entwicklungs-GmbH.
# All rights reserved.
#
###############################################################################
#
# DESCRIPTION:
#
# Tcl code sourced into IB by the main interp of the joyDeveloper.palette.
#
###############################################################################

#######################################################################
# 			 Inspector Stuff
#######################################################################

objc:teach ITKTclInterp {

- inspectorClassName { @ ITKTclInterpInspector }

- connectInspectorClassName { @ ITKConnectInspector }

- editorClassName { @ ITKTclInterpEditor }

}

#######################################################################
# 			Images
#######################################################################

objc:teach ITKTclInterp {

- id imageForViewer {
    set bundle [NSBundle bundleForClass: itkDeveloper]
    set file [$bundle pathForImageResource: [@ interpreter]]
    if {$file != "nil"} {
        return [[NSImage alloc] initByReferencingFile: $file]
    }
    return nil
}

}

#######################################################################
# 			Interpreter init files
#######################################################################

objc:declare STR {tmpnam tcl_tmpnam} {STR}

objc:teach $self {

- void willSaveDocument: {{NSNotification *} note} {
    set document [$note object]
    upvar #0 ${document}(tmpPath) tmpPath
    set tclFiles [glob -nocomplain -- \
		       [file join [* [$document documentPath]] *.tcl]]
    if {$tclFiles != ""} {
        set tmpPath [tmpnam [NULL]]
	file mkdir $tmpPath
	eval [list file copy --] $tclFiles [list $tmpPath]
    }
}

- void didSaveDocument: {{NSNotification *} note} {
    set document [$note object]
    upvar #0 ${document}(tmpPath) tmpPath
    if {[info exists tmpPath]} {
	set tclFiles [glob -nocomplain [file join $tmpPath *]]
	eval [list file copy --] $tclFiles [list [* [$document documentPath]]]
	file delete -force $tmpPath
	unset tmpPath
    }
}

- void openTutorial: sender {
    global tcl_platform
    set workspace [NSWorkspace sharedWorkspace]
    if {$tcl_platform(platform) == "windows"} {
        if {![$workspace openFile: [@ $env(NEXT_ROOT)/LocalLibrary/Joy/Documentation/0_Tutorial.html]]} {
            $workspace openFile: [@ $env(NEXT_ROOT)/LocalLibrary/Joy/Documentation/0_Tutorial.rtfd]
        }
    } elseif {$tcl_platform(os) == "OPENSTEP"} {
        $workspace openFile: [@ /LocalLibrary/Joy/Documentation/0_Tutorial.rtfd]
    } else {
        $workspace openFile: [@ /Local/Library/Joy/Documentation/0_Tutorial.html]
    }
}

}

$defaultCenter addObserver: $self \
		  selector: willSaveDocument: \
		      name: [@ IBWillSaveDocumentNotification] \
		    object: nil
$defaultCenter addObserver: $self \
		  selector: didSaveDocument: \
		      name: [@ IBDidSaveDocumentNotification] \
		    object: nil

objc:teach ITKTclInterp {

- void unparseInitFile: file {
    global NSApp
    set f [open [* $file] w]
    puts $f "# Joy interpreter init file created automatically, \
             [clock format [clock seconds] -format %c]\n"
    set conns [[$NSApp activeDocument] itkConnectorsForSource: $self]
    set objVars self
    set objs $self
    set otherVars ""
    objc:foreach conn $conns {
	set var [* [$conn sourceAspect]]
	if {[* [$conn destinationAspect]] == "self"} {
	    if {![lcontain $objVars $var]} {
		lappend objVars $var
		lappend objs [$conn destination]
	    }
	} else {
	    if {![lcontain $otherVars $var]} {
		lappend otherVars $var
	    }
	}
    }
    set globals [union $objVars $otherVars]
    lvarpop globals [lsearch -exact $globals self]
    foreach v $objVars o $objs {
	puts $f "objc:teach \$$v {\n"
	catch {$o unparseSkillsTo: $f globals: $globals}
	puts $f "}\n"
    }
    close $f
}

}

objc:teach NSObject {

- void unparseSkill: {TclObj desc} to: {TclObj f} globals: {TclObj globals} {
    puts $f "$desc {"
    puts $f "    global $globals"
    puts $f "\n}\n"
}

- void unparseSkillsTo: {TclObj f} globals: {TclObj globals} {}

}

objc:teach {NSControl NSActionCell} {

- void unparseSkillsTo: {TclObj f} globals: {TclObj globals} {
    super unparseSkillsTo: $f globals: $globals
    $self unparseSkill: "- void action: sender" to: $f globals: $globals
    if {[$self respondsToSelector: doubleAction]} {
	$self unparseSkill: "- void doubleAction: sender" \
			to: $f \
		   globals: $globals
    }
}

}

#######################################################################
# 			    Show/Hide Command Window
########################################################################

objc:teach $self {

- void willBeginTestingInterface: {{NSNotification *} note} {
    iset visible [[$self commandWindow] isVisible]
}

- void didEndTestingInterface: {{NSNotification *} note} {
    if {[iset visible]} {
	[$self commandWindow] makeKeyAndOrderFront: $self
    } else {
	[$self commandWindow] close
    }
}

}

$defaultCenter addObserver: $self \
		  selector: willBeginTestingInterface: \
		      name: [@ IBWillBeginTestingInterfaceNotification] \
		    object: nil

$defaultCenter addObserver: $self \
		  selector: didEndTestingInterface: \
		      name: [@ IBDidEndTestingInterfaceNotification] \
		    object: nil

#######################################################################
# 			    Menu entries
#######################################################################

objc:teach $NSApp {

- void newJoyApplication: sender {
    global tcl_platform
    # Determine the new application's interface style
    set interfaceStyle [[[[IBDocument alloc] init] autorelease] interfaceStyle]
    if {$interfaceStyle == "[NSNextStepInterfaceStyle]"} {
        set template next
    } elseif {$interfaceStyle == "[NSWindows95InterfaceStyle]"} {
        set template windows
    } else {
        set template mac
    }

    # Load template nib
    set document [[IBDocument alloc] initFromPath: [[NSBundle bundleForClass: itkDeveloper] pathForResource: [@ NewJoyApplication-$template] ofType: [@ nib]]]

    # Make it a brand new untitled and unedited document
    $document setUntitled: [YES]
    $document setDocumentPath: [@ UNTITLED]
    $document edit
    if {[$document respondsToSelector: setDocumentEdited:]} {
        $document setDocumentEdited: [NO]
    }
}

}


# Create Joy menu ...
set _joyMenu [[NSMenu alloc] initWithTitle: [@ Joy]]
# ... and make it submenu of the mainMenu
set _item [[$NSApp mainMenu] \
             insertItemWithTitle: [@ Joy] \
                          action: 0 \
                   keyEquivalent: [@ ""] \
                         atIndex: 5]
[$NSApp mainMenu] setSubmenu: $_joyMenu forItem: $_item
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

# Add the entries in Joy menu.

# Create Info menu ...
set _infoMenu [[NSMenu alloc] initWithTitle: [@ Info]]
# ... and make it submenu of the Joy menu
set _item [$_joyMenu \
             addItemWithTitle: [@ Info] \
		       action: 0 \
		keyEquivalent: [@ ""]]
$_joyMenu setSubmenu: $_infoMenu forItem: $_item
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

if {$tcl_platform(os) == "OPENSTEP"} {
    set _infoPanelTitle "Info Panel..."
} else {
    set _infoPanelTitle "About Joy..."
}	
set _item [$_infoMenu \
		addItemWithTitle: [@ $_infoPanelTitle] \
			  action: showInfoPanel: \
		   keyEquivalent: [@ ""]]
$_item setTarget: [JoyOrderController sharedController]
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0
unset _infoPanelTitle

set _item [$_infoMenu \
		addItemWithTitle: [@ "Order..."] \
			  action: showOrderPanel: \
		   keyEquivalent: [@ ""]]
$_item setTarget: [JoyOrderController sharedController]
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_infoMenu \
		addItemWithTitle: [@ "Register Host..."] \
			  action: showHostRegistrationPanel: \
		   keyEquivalent: [@ ""]]
$_item setTarget: [JoyHostRegistrationController sharedController]
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_infoMenu \
		addItemWithTitle: [@ "License..."] \
			  action: runRegistrationPanel: \
		   keyEquivalent: [@ ""]]
$_item setTarget: [JoyRegistrationController sharedController]
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_infoMenu \
		addItemWithTitle: [@ "License Status..."] \
			  action: showRegistrationStatus: \
		   keyEquivalent: [@ ""]]
$_item setTarget: [JoyRegistrationController sharedController]
$_item setEnabled: [YES]
$_item setMnemonicLocation: 8

set _item [$_infoMenu \
		addItemWithTitle: [@ "Help..."] \
			  action: openTutorial: \
		   keyEquivalent: [@ ""]]
$_item setTarget: $self
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_joyMenu \
		addItemWithTitle: [@ "Command Window..."] \
			  action: showCommandWindow: \
		   keyEquivalent: [@ J]]
$_item setTarget: $mainInterp
$_item setEnabled: [YES]
$_item setMnemonicLocation: 8

set _item [$_joyMenu \
		addItemWithTitle: [@ "Save Commands..."] \
			  action: saveCommands: \
		   keyEquivalent: [@ ""]]
$_item setTarget: $mainInterp
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_joyMenu \
		addItemWithTitle: [@ "Clear Command Window"] \
			  action: clearCommandWindow: \
		   keyEquivalent: [@ k]]
$_item setTarget: $mainInterp
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_joyMenu \
		addItemWithTitle: [@ "New Joy Application"] \
			  action: newJoyApplication: \
		   keyEquivalent: [@ ""]]
$_item setTarget: $NSApp
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_joyMenu \
		addItemWithTitle: [@ "Save Nib as App..."] \
			  action: 0 \
		   keyEquivalent: [@ A]]
$_item setMnemonicLocation: 12

unset _item
unset _infoMenu
unset _joyMenu

#######################################################################
# 			    User Startup
########################################################################

if {[file readable ~/.itkDeveloperInit.tcl]} {
    if {[catch {source ~/.itkDeveloperInit.tcl} msg]} {
	puts ".itkDeveloperInit.tcl: $msg"
    }
}
